/*
 * Copyright (c) 2014 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Wed Mar  5 2014
 */
#include "acoptitrack.h"

#include <err.h>
#include <stdio.h>

#include "optitrack_c_types.h"
#include "codels.h"


/* --- opti_update_ports --------------------------------------------------- */

genom_event
opti_create_ports(const struct natnet_descr *descr,
                  const optitrack_bodies *bodies, genom_context self)
{
  const struct natnet_bodydescr *body;
  genom_event e;
  uint32_t i;

  for(i = 0, body = descr->body; i < descr->nbodies; i++, body++) {
    if (bodies->data(body->safename, self)) continue;

    e = bodies->open(body->safename, self);
    if (e != genom_ok) {
      warnx("failed to export body '%s'", body->safename);
      return e;
    }
  }

  return genom_ok;
}


/* --- opti_update_ports --------------------------------------------------- */

genom_event
opti_update_ports(const struct natnet_frame *f,
                  optitrack_server_s *server,
                  const optitrack_bodies *bodies, FILE *log, genom_context self)
{
  or_pose_estimator_state *state;
  const struct natnet_body *body;
  struct timeval tv;
  double qx, qy, qz, qw;
  genom_event e;
  uint32_t i;

  for(i=0, body=f->body; i<f->nbodies; i++, body++) {
    if (body->descr == NULL) {
      warnx("body %d has gone\n", f->body[i].id);
      server->descr = NULL; /* refetch descriptions */
      return genom_ok;
    }

    state = bodies->data(body->descr->safename, self);
    if (!state) {
      warnx("body %s cannot be published \n", body->descr->safename);
      return optitrack_e_publish(self);
    }

    /* local time for now */
    gettimeofday(&tv, NULL);
    state->ts.sec = tv.tv_sec;
    state->ts.nsec = tv.tv_usec * 1000.;

    state->ref = or_pose_estimator_GLOBAL_FRAME;
    state->model = or_pose_estimator_MOTION_UNKNOWN;
    state->pos._present = (body->error > 0.);

    /* remap optitrack coordinates into something more usual (Z up) */
    state->pos._value.x = body->pos->z;
    state->pos._value.y = body->pos->x;
    state->pos._value.z = body->pos->y;

    qw = body->pos->qw;
    qx = body->pos->qz;
    qy = body->pos->qx;
    qz = body->pos->qy;

    state->pos._value.qw = qw;
    state->pos._value.qx = qx;
    state->pos._value.qy = qy;
    state->pos._value.qz = qz;

    /* ignore the rest for now */
    state->pos_cov._present = 0;
    state->vel._present = 0;
    state->vel_cov._present = 0;
    state->acc._present = 0;
    state->acc_cov._present = 0;

    e = bodies->write(body->descr->safename, self);
    if (e != genom_ok) {
      warnx("body %s cannot be published \n", body->descr->safename);
      return e;
    }

    if (log && state->pos._present) {
      fprintf(log, "%s %d %d   %g %g %g   %g %g %g %g   %g\n",
              body->descr->safename,
              state->ts.sec, state->ts.nsec,
              state->pos._value.x,
              state->pos._value.y,
              state->pos._value.z,
              state->pos._value.qw,
              state->pos._value.qx,
              state->pos._value.qy,
              state->pos._value.qz,
              body->error);
    }
  }

  return genom_ok;
}
